// E.I.S. / ESA modlib for COMRADE and HardScale
  //
  // Version History:
  // ----------------
  // 100826: BT: intial release

`timescale 1ns / 1ns

module inreg
         #(parameter
                      WA                   = 32,
                      WR                   = WA,
                      SIGN                 = 0, // dummy
                      IOADDR               = 1,
                      IOAWIDTH             = 4,
                      STATIC_CT            = 0,
                      START_CTRL_IN        = 0,
                      DEPTH                = 0, // dummy
                      QDEPTH               = 0,
                      TQDEPTH              = 0,
                      DATA_SPECULATION     = 0,
                      SPECULATION_BOUNDARY = 0,
                      PSEUDO_AT            = 0
          )(
           input  wire [(WA-1):0]       A, // IODIN
           output reg  [(WR-1):0]       R,
           input  wire                  RESET,
           input  wire                  CLK,
           input  wire                  CE,
           input  wire                  CONTROLLER_CANCEL,
           output wire                  TOKENS_EMPTY,
           output wire                  CURRENT_TOKEN,
           output wire                  CURRENT_TOKEN_PSEUDO,
           input  wire [(IOAWIDTH-1):0] IOA,
           input  wire                  IOSEL,
           input  wire                  START,
           output wire                  START_ACK,
           input  wire                  START_CTRL,
           output wire                  START_CTRL_ACK,
           input  wire                  START_CTRL_PSEUDO,
           input  wire                  CANCEL,
           output wire                  CANCEL_ACK,
           output wire                  CANCEL_STATE,
           input  wire                  CANCEL_STATE_ACK,
           output wire                  CANCEL_STATE_CTRL,
           input  wire                  CANCEL_STATE_CTRL_ACK,
           output reg                   RESULT_READY,
           input  wire                  RESULT_READY_ACK);

  wire  computation_counter = 1'b0; // for PDF counter obersvation only
  wire  cancel_counter = 1'b0;      // for PDF counter obersvation only

  assign TOKENS_EMPTY = ~RESULT_READY;
  assign CURRENT_TOKEN = 1'b0;
  assign CURRENT_TOKEN_PSEUDO = 1'b0;

  // immediately start
  assign START_ACK = START && ~RESULT_READY;
  assign START_CTRL_ACK = 1'b0;
  // never cancel
  assign CANCEL_STATE = 1'b0;
  assign CANCEL_STATE_CTRL = 1'b0;
  assign CANCEL_ACK = 1'b0;
  
  // R
  always @(posedge CLK, posedge RESET) begin
    if (RESET) begin
      R <= 1'b0;
    end
    else begin
      if (IOSEL && (IOA == IOADDR) && START && START_ACK) begin
        R <= A;
      end
    end
  end

  // RESULT_READY
  always @(posedge CLK, posedge RESET) begin
    if (RESET) begin
      RESULT_READY <= 1'b0;
    end
    else begin
      if (IOSEL && (IOA == IOADDR) && START && START_ACK) begin
        RESULT_READY <= 1'b1;
      end else if (RESULT_READY_ACK) begin
        RESULT_READY <= 1'b0;
      end
    end
  end

endmodule

