// E.I.S. / ESA modlib for COMRADE and HardScale
  //
  // Version History:
  // ----------------
  // 100826: BT: intial release

`timescale 1ns / 1ns

module irqreg
         #(parameter
                      WA                   = 32,
                      WR                   = WA,
                      SIGN                 = 0, // dummy
                      IOADDR               = 0, // IRQREG ist always at I/O address 0
                      IOAWIDTH             = 4,
                      STATIC_CT            = 0,
                      START_CTRL_IN        = 0,
                      DEPTH                = 0, // dummy
                      QDEPTH               = 0,
                      TQDEPTH              = 0, // dummy
                      DATA_SPECULATION     = 0,
                      SPECULATION_BOUNDARY = 0,
                      PSEUDO_AT            = 0
          )(
           input  wire [(WA-1):0]        A,
           output reg  [(WR-1):0]        R,
           input  wire                   RESET,
           input  wire                   CLK,
           input  wire                   CE,
           input  wire                   CONTROLLER_CANCEL,
           output wire                   TOKENS_EMPTY,
           output wire                   CURRENT_TOKEN,
           output wire                   CURRENT_TOKEN_PSEUDO,
           input  wire [(IOAWIDTH-1):0]  IOA,
           input  wire                   IOSEL,
           output wire [(WR-1):0]        IODOUT,
           output wire                   IRQ,
           input  wire                   START,
           output wire                   START_ACK,
           input  wire                   START_CTRL,
           output wire                   START_CTRL_ACK,
           input  wire                   START_CTRL_PSEUDO,
           input  wire                   CANCEL,
           output wire                   CANCEL_ACK,
           output wire                   CANCEL_STATE,
           input  wire                   CANCEL_STATE_ACK,
           output wire                   CANCEL_STATE_CTRL,
           input  wire                   CANCEL_STATE_CTRL_ACK,
           output reg                    RESULT_READY,
           input  wire                   RESULT_READY_ACK);


  wire  computation_counter = 1'b0; // for PDF counter obersvation only
  wire  cancel_counter = 1'b0;      // for PDF counter obersvation only

  assign TOKENS_EMPTY = ~RESULT_READY;
  assign CURRENT_TOKEN = 1'b0;
  assign CURRENT_TOKEN_PSEUDO = 1'b0;

  // this must be combined with all other OUTREGS using a wide-OR.
  assign IODOUT = (IOSEL && (IOA == IOADDR)) ? R : 0;

  // fast IRQ (rising flank combinatorial)
  assign IRQ = START && START_ACK && CE;

  // always starts immediately
  assign START_ACK = START;
  assign START_CTRL_ACK = 1'b0;
  // never cancelled
  assign CANCEL_STATE = 0;
  assign CANCEL_STATE_CTRL = 0;
  assign CANCEL_ACK = 0;

  always @(posedge CLK, posedge RESET) begin
    if (RESET) begin
      R <= 0;
      RESULT_READY <= 0;
    end else begin
      if (RESULT_READY) begin
        RESULT_READY <= 0; // reset RESULT_READY after 1 cycle
      end else if (START && START_ACK && CE) begin // writing reg sets IRQ
        R <= A;
        RESULT_READY <= 1;
      end
    end
  end

endmodule

