// E.I.S. / ESA modlib for COMRADE and HardScale
  //
  // Description:
  // ----------------
  // Universal Register function
  // NOP (No Operation) with output queue and shift register mechanism
  // This nop_sink is exactly a nop, but tells the compiler to connect its
  // I/O such that ATs are just discarded (token sink).
  //
  // Version History:
  // ----------------
  // 100826: BT: intial release

`timescale 1ns / 1ns

module nop_sink
         #(parameter
                      WA                   = 32,
                      WB                   = 1,  // NIN
                      NIN                  = WB,
                      WR                   = WA, // 1 for logic operations
                      DEPTH                = 0,
                      QDEPTH               = 16,
                      TQDEPTH              = 16,
                      STATIC_CT            = 0,
                      START_CTRL_IN        = 0,
                      DATA_SPECULATION     = 0,
                      SPECULATION_BOUNDARY = 0,
                      SIGN                 = 0,
                      PSEUDO_AT            = 1
          )(
           input  wire [(WA*NIN-1):0]    A,
           output wire [(WR-1)    :0]    R,
           input  wire                   RESET,
           input  wire                   CLK,
           input  wire                   CE,
           input  wire                   CONTROLLER_CANCEL,
           output wire                   TOKENS_EMPTY,
           output wire                   CURRENT_TOKEN,
           output wire                   CURRENT_TOKEN_PSEUDO,
           input  wire                   START,
           output wire                   START_ACK,
           input  wire                   START_CTRL,
           output wire                   START_CTRL_ACK,
           input  wire                   START_CTRL_PSEUDO,
           input  wire                   CANCEL,
           output wire                   CANCEL_ACK,
           output wire                   CANCEL_STATE,
           input  wire                   CANCEL_STATE_ACK,
           output wire                   CANCEL_STATE_CTRL,
           input  wire                   CANCEL_STATE_CTRL_ACK,
           output wire                   RESULT_READY,
           input  wire                   RESULT_READY_ACK);
nop #(
  .WA(WA),
  .WB(WB),
  .NIN(NIN),
  .WR(WR),
  .DEPTH(DEPTH),
  .QDEPTH(QDEPTH),
  .DATA_SPECULATION(DATA_SPECULATION),
  .SPECULATION_BOUNDARY(SPECULATION_BOUNDARY),
  .STATIC_CT(STATIC_CT == 1),
  .START_CTRL_IN(START_CTRL_IN == 1),
  .SIGN(SIGN),
  .PSEUDO_AT(PSEUDO_AT == 1)
) Nop (
  .A(A),
  .R(R),
  .RESET(RESET),
  .CLK(CLK),
  .CE(CE),
  .CONTROLLER_CANCEL(CONTROLLER_CANCEL),
  .TOKENS_EMPTY(TOKENS_EMPTY),
  .CURRENT_TOKEN(CURRENT_TOKEN),
  .CURRENT_TOKEN_PSEUDO(CURRENT_TOKEN_PSEUDO),
  .START(START),
  .START_ACK(START_ACK),
  .START_CTRL(START_CTRL),
  .START_CTRL_ACK(START_CTRL_ACK),
  .START_CTRL_PSEUDO(START_CTRL_PSEUDO),
  .CANCEL(CANCEL),
  .CANCEL_ACK(CANCEL_ACK),
  .CANCEL_STATE(CANCEL_STATE),
  .CANCEL_STATE_ACK(CANCEL_STATE_ACK),
  .CANCEL_STATE_CTRL(CANCEL_STATE_CTRL),
  .CANCEL_STATE_CTRL_ACK(CANCEL_STATE_CTRL_ACK),
  .RESULT_READY(RESULT_READY),
  .RESULT_READY_ACK(RESULT_READY_ACK)
);

endmodule
