// E.I.S. / ESA modlib for COMRADE and HardScale
  //
  //// Description:
  // ----------------
  // Universal shift register
  //
  // Version History:
  // ----------------
  // 100826: BT: intial release

`timescale 1ns / 1ns

module shiftreg
         #(parameter
                      WA    = 32,
                      WR    = WA,
                      DEPTH = 16
          )(
           input       [(WA-1):0] A,
           output      [(WR-1):0] R,
           input                  RESET,
           input                  CLK,
           input                  CE,
           input                  START,
           output                 RESULT_READY);



generate

if(DEPTH == 0) begin
  assign R = A;
  assign RESULT_READY = START;
end

else begin
  reg  [(WR-1)   :0] stages [0:(DEPTH-1)];
  reg  [(DEPTH-1):0] valid_int;
  integer i = 0;

  assign R = stages[DEPTH-1];
  assign RESULT_READY = valid_int[DEPTH-1];

  always @(posedge CLK, posedge RESET) begin
      if (RESET) begin
        for (i=0; i < DEPTH; i = i + 1) begin
          stages[i] <= 0;
        end
        valid_int <= 0;
      end
      else if (CE) begin
        stages[0] <= A;
        for (i=1; i < DEPTH; i = i + 1) begin
          stages[i] <= stages[i-1];
        end
        valid_int <= (valid_int << 1) | START;
      end
  end

end
endgenerate

endmodule
