// E.I.S. / ESA modlib for COMRADE and HardScale
  //
  //// Description:
  // ----------------
  // Module used for token forwarding (no data flow).
  //
  // Version History:
  // ----------------
  // 100826: BT: intial release

`timescale 1ns / 1ns

module token_nop
         #(parameter
                      STATIC_CT            = 0,
                      START_CTRL_IN        = 0,
                      DATA_SPECULATION     = 0,
                      SPECULATION_BOUNDARY = 0,
                      PSEUDO_AT            = 0,
                      WR                   = 1,
                      DEPTH                = 0,
                      TQDEPTH              = 0,
                      QDEPTH               = 0
          )(
           input  wire               RESET,
           input  wire               CLK,
           input  wire               CE,
           input  wire               CONTROLLER_CANCEL,
           output wire               TOKENS_EMPTY,
           output wire               CURRENT_TOKEN,
           output wire               CURRENT_TOKEN_PSEUDO,
           input  wire               START,
           output wire               START_ACK,
           input  wire               START_CTRL,
           output wire               START_CTRL_ACK,
           input  wire               START_CTRL_PSEUDO,
           input  wire               CANCEL,
           output reg                CANCEL_ACK,
           output wire               CANCEL_STATE,
           input  wire               CANCEL_STATE_ACK,
           output wire               CANCEL_STATE_CTRL,
           input  wire               CANCEL_STATE_CTRL_ACK,
           output wire               RESULT_READY,
           input  wire               RESULT_READY_ACK);
           
  // define maximum LATENCY of this module
  localparam LATENCY          = 1;
  // CANCEL BITS depending on LATENCY and shift register DEPTH
  localparam CANCEL_BITS      = min_1(ceil_log2(LATENCY+DEPTH-1));
  // QDEPTH_INT has to be at least 16
  localparam QDEPTH_INT       = min_16(QDEPTH);
  // set QBITS accordingly
  localparam QBITS            = (QDEPTH == 0) ? 0 : ceil_log2(QDEPTH_INT-1);
  // TQDEPTH_INT has to be at least 16
  localparam TQDEPTH_INT      = min_16(TQDEPTH);
  // set TQBITS accordingly
  localparam TQBITS           = (TQDEPTH == 0) ? 0 : ceil_log2(TQDEPTH_INT-1);
  // COMPUTATION_BITS depending  on LATENCY shift register DEPTH and queue QDEPTH
  localparam COMPUTATION_BITS = min_1(ceil_log2(LATENCY+DEPTH+QDEPTH_INT-1));

  // dummy wire for handshaking_assigns_verilog_only.v
  wire R;

  `include "handshaking_assigns_verilog_only.v"
  
  
  assign DATA_SHIFTREG_TO_QUEUE = {(WR){1'b0}};


  // shift register instantiation
  shiftreg #(.WA(1), .WR(1), .DEPTH(DEPTH)) sr (
    .A(1'b0),
    .RESET(RESET),
    .CLK(CLK),
    .CE(CE_SHIFTREG),
    .START(START_SHIFTREG),
    .RESULT_READY(RESULT_READY_SHIFTREG));
    
  `include "functions.v"

endmodule
