// E.I.S. / ESA modlib for COMRADE and HardScale
  //
  //// Description:
  // ----------------
  // variable leftshift operation
  // with output queue and shift register mechanism
  //
  // Version History:
  // ----------------
  // 100826: BT: intial release

`timescale 1ns / 1ns

module vsl
         #(parameter
                      WA                   = 32,
                      WB                   = 5,  // NUMBER OF SHIFTS
                      WR                   = WA,
                      DEPTH                = 0,
                      QDEPTH               = 16,
                      TQDEPTH              = 16,
                      STATIC_CT            = 0,
                      START_CTRL_IN        = 0,
                      DATA_SPECULATION     = 0,
                      SPECULATION_BOUNDARY = 0,
                      PSEUDO_AT            = 0,
                      SIGN                 = 0
          )(
           input  wire    [(WA-1):0] A,
           input  wire    [(WB-1):0] B,
           output wire    [(WR-1):0] R,
           input  wire               RESET,
           input  wire               CLK,
           input  wire               CE,
           input  wire               CONTROLLER_CANCEL,
           output wire               TOKENS_EMPTY,
           output wire               CURRENT_TOKEN,
           output wire               CURRENT_TOKEN_PSEUDO,
           input  wire               START,
           output wire               START_ACK,
           input  wire               START_CTRL,
           output wire               START_CTRL_ACK,
           input  wire               START_CTRL_PSEUDO,
           input  wire               CANCEL,
           output reg                CANCEL_ACK,
           output wire               CANCEL_STATE,
           input  wire               CANCEL_STATE_ACK,
           output wire               CANCEL_STATE_CTRL,
           input  wire               CANCEL_STATE_CTRL_ACK,
           output wire               RESULT_READY,
           input  wire               RESULT_READY_ACK);

  // define LATENCY of this module
  localparam LATENCY          = 1;
  // CANCEL BITS depending on LATENCY and shift register DEPTH
  localparam CANCEL_BITS      = min_1(ceil_log2(LATENCY+DEPTH-1));
  // QDEPTH_INT has to be at least 16
  localparam QDEPTH_INT       = min_16(QDEPTH);
  // set QBITS accordingly
  localparam QBITS            = (QDEPTH == 0) ? 0 : ceil_log2(QDEPTH_INT-1);
  // TQDEPTH_INT has to be at least 16
  localparam TQDEPTH_INT      = min_16(TQDEPTH);
  // set TQBITS accordingly
  localparam TQBITS           = (TQDEPTH == 0) ? 0 : ceil_log2(TQDEPTH_INT-1);
  // COMPUTATION_BITS depending  on LATENCY shift register DEPTH and queue QDEPTH
  localparam COMPUTATION_BITS = min_1(ceil_log2(LATENCY+DEPTH+QDEPTH_INT-1));

  `include "handshaking_assigns_verilog_only.v"
  
  wire [WA-1:0] r_int;
  
  generate
  if (SIGN == 1) begin
    assign r_int = A <<< B;
  end
  else begin
    assign r_int = A << B;
  end
  endgenerate


/****************** alternative code if variable shifts not allowed ***********
  integer i;
  reg [WA-1:0] r_int;
  generate
  if (SIGN == 1) begin
    wire [(WA*2)-4:0] in_vec;
    assign in_vec = {A[WA-2:0], {(WA-2){1'b0}}};
    always @(*) begin
      r_int[WA-2:0] = {(WA-1){1'b0}};
      for (i = 0; i < WA-1; i = i + 1) begin
        if (B == i) begin
         r_int[WA-2:0] = in_vec[(WA-2-i) +: (WA-1)];
        end
      end
      r_int[WA-1] = A[WA-1];
    end
  end
  else begin
    wire [(WA*2)-2:0] in_vec;
    assign in_vec = {A[WA-1:0], {(WA-1){1'b0}}};
    always @(*) begin
      r_int[WA-1:0] = {(WA){1'b0}};
      for (i = 0; i < WA; i = i + 1) begin
        if (B == i) begin
          r_int[WA-1:0] = in_vec[(WA-1-i) +: WA];
        end
      end
    end
  end
  endgenerate
******************************************************************************/
/****************** Florians Code ***************************
  genvar i;
  wire [WA-1:0] d [0:WA-1];
  assign R = d[WA-1];
  assign d[0] = A;


        generate
        for (i = 1; i < WA; i = i + 1) begin
          assign d[i] = (B==i) ? A << i : d[i-1];
        end
        endgenerate
************************************************************/
  // shift register instantiation
  shiftreg #(.WA(WR), .WR(WR), .DEPTH(DEPTH)) sr (
    .A(r_int),
    .R(DATA_SHIFTREG_TO_QUEUE),
    .RESET(RESET),
    .CLK(CLK),
    .CE(CE_SHIFTREG),
    .START(START_SHIFTREG),
    .RESULT_READY(RESULT_READY_SHIFTREG));


  `include "functions.v"

endmodule
